// dump/main.cpp
// Part of SalvageNTFS
// Copyright (C) Will Glynn
//////////////////////////////////////////////////////////////////////////////
// LICENSE
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License (GPL)
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// To read the license please visit http://www.gnu.org/copyleft/gpl.html

#include <stdio.h>

#include "ntfsvolume.h"
#include "ntfsutil.h"
#include "ntfsstream.h"
#include "ntfsfile.h"
#include "ntfsvolumeinfo.h"

// Test function
void test(int fd, NT64 offset, NT64 length)
{
	NTFSVolume v;
	v.attach(fd);

	// Scan first sector for an NTFS boot sector
	//v.analyzeBootSector(0);

	// Prep testing environment
	NTFSStream * s = new NTFSStream;
	NTFSFile * nf;
	const NTFSVolumeInfo * vi = v.getVolInfo();

	NT64 records = offset;
	NT64 maxRecords = (length ? offset + length : 0);

	if (length)
	{
		printf(QString("\nDumping %1 record(s) from %2.\n").arg(toHex(length)).arg(toHex(offset)));
	}
	else
	{
		printf(QString("\nDumping records from %1 to the end of the volume.\n").arg(toHex(offset)));
	}

	bool readSomething = false;
	while ((length == 0) || (records < maxRecords))
	{
		if (!v.streamData(s, records * vi->bytesPerRecord, vi->bytesPerRecord))
		{
			break;
		}

		nf = new NTFSFile();
		if (nf->read(s))
		{
			printf("%s\n", nf->toString().latin1());
			fflush(stdout);
			readSomething = true;
		}
		delete nf;

		records ++;
	}

	if (!readSomething)
	{
		printf("No FILE records found.\n");
	}
}

// Entry point
int main(int argc, char ** argv)
{
	printf("SalvageNTFS Dump\nWritten by Will Glynn\n\n");

	if (argc < 2 || argc > 4)
	{
		printf("Usage: %s volume [offset [length]]\n", argv[0]);
		return 1;
	}

	FILE * f = fopen(argv[1], "r");
	if (!f)
	{
		printf("Unable to open %s for reading.\n", argv[1]);
		return 1;
	}

	NT64 offset = 0, length = 0;
	if (argc > 2)
	{
		offset = toNum(argv[2]);
	}
	if (argc > 3)
	{
		length = toNum(argv[3]);
	}

	test(fileno(f), offset, length);

	return 0;
}
