// ntfsindexentry.cpp
// Part of SalvageNTFS
// Copyright (C) Will Glynn
//////////////////////////////////////////////////////////////////////////////
// LICENSE
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License (GPL)
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// To read the license please visit http://www.gnu.org/copyleft/gpl.html

#include "ntfsindexentry.h"
#include "ntfsutil.h"

NTFSIndexEntry::NTFSIndexEntry()
{
	fileRef = 0;
	flags = 0;
	data = NULL;
}

NTFSIndexEntry::~NTFSIndexEntry()
{
	if (data) delete data;
}

QString NTFSIndexEntry::toString() const
{
	return QString("%1: %2").arg(toHexRef(fileRef)).arg(data->toString());
}

bool NTFSIndexEntry::read(NTFSStream * s)
{
	// Sanity check
	if (s == NULL)
	{
		BUG("Passed NULL stream.");
		return false;
	}

	NT32 initialOffset = s->getOffset();

	// Read fields
	fileRef = s->read64();
	NT16 recordSize = s->read16();
	NT16 filenameOffset = s->read16();
	flags = s->read16();

	//TRACE(QString("filenameOffset = %1, flags = %2").arg(toHex(filenameOffset)).arg(toHex(flags)));

	// Ignore padding
	s->read16();

	// Are we an index entry, or a terminating header?
	if (flags & 0x02)
	{
		// THE TERMINATOR
		// ;-)
		data = NULL;
		return true;
	}
	else
	{
		// Read filename data
		data = new NTFSAttributeDataFilename;
		if (!data->read(s))
		{
			// :-(
			SERROR("Failed to read filename data.", s);
			delete data;
			return false;
		}
		
		// Seek to end of record
		s->seek(initialOffset + recordSize);

		return true;
	}
}
