// ntfsrunlist.h
// Part of SalvageNTFS
// Copyright (C) Will Glynn
//////////////////////////////////////////////////////////////////////////////
// LICENSE
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License (GPL)
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// To read the license please visit http://www.gnu.org/copyleft/gpl.html

#ifndef NTFSRUNLIST_H
#define NTFSRUNLIST_H

#include "ntfs.h"
#include "ntfsstream.h"

#include <qptrlist.h>

class NTFSVolume;

class NTFSRunList
{
public:
    NTFSRunList();
    ~NTFSRunList();
	void copy(NTFSRunList * src);
	bool read(NTFSStream * s);
	QString toString();

	bool first(NT64 * lcn, NT64 * length);
	bool next(NT64 * lcn, NT64 * length);

	// De-references a data run and puts it in a stream
	bool copyDataToStream(NTFSVolume * v, NTFSStream * s, NT64 len, NT16 flags = 0);

	// De-references a data run and writes it directly to a file descriptor
	bool copyDataToFile(NTFSVolume * v, int fd, NT64 len, NT16 flags = 0);

	// Appends a runlist to the current runlist
	void append(NTFSRunList * data);

	// Inserts a sparse run length clusters long at vcn
	void addSparseRun(NT64 vcn, NT64 length);

protected:
	struct RUNLIST {
		NT64 vcn;
		NT64 lcn;
		NT64 length;
		bool sparse;
	};

	QPtrList<RUNLIST> m_runList;
};

#endif
