// ntfsutil.h
// Part of SalvageNTFS
// Copyright (C) Will Glynn
//////////////////////////////////////////////////////////////////////////////
// LICENSE
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License (GPL)
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// To read the license please visit http://www.gnu.org/copyleft/gpl.html

#include <qstring.h>
#include <time.h>
#include "ntfs.h"

// Fixed-length output
QString to8Hex(NT8 n);
QString to16Hex(NT16 n);
QString to32Hex(NT32 n);
QString to64Hex(NT64 n);

// Variable-length output
QString toHex(NT32 n);
QString toHex(NT64 n);

inline QString toHex(NT16 n) { return toHex((NT32) n); };
inline QString toHex(NT8 n) { return to8Hex(n); };

// Intelligent formatting
QString toHexRef(NT64 refnum);

// Variable-form input
NT64 toNum(QString string);

// Format conversion
time_t time64ToUnix(NT64 timestamp);

// Usefulness
#define MIN(a,b) (a > b ? b : a)
#define MAX(a,b) (a > b ? a : b)

// Debugging
#define BUG(o) bugTrace(__FILE__, __LINE__, __FUNCTION__, "BUG: ", o, NULL)
#define ERROR(o) nTrace(__FILE__, __LINE__, __FUNCTION__, "Error: ", o, NULL)
#define WARN(o) nTrace(__FILE__, __LINE__, __FUNCTION__, "Warning: ", o, NULL)
#define TRACE(o) nTrace(__FILE__, __LINE__, __FUNCTION__, "", o, NULL)
#define SBUG(o,s) nTrace(__FILE__, __LINE__, __FUNCTION__, "BUG: ", o, s)
#define SERROR(o,s) nTrace(__FILE__, __LINE__, __FUNCTION__, "Error: ", o, s)
#define SWARN(o,s) nTrace(__FILE__, __LINE__, __FUNCTION__, "Warning: ", o, s)
#define STRACE(o,s) nTrace(__FILE__, __LINE__, __FUNCTION__, "", o, s)
class NTFSStream;
void bugTrace(QString file, int line, QString function, QString prefix, QString message, const NTFSStream * s);
void nTrace(QString file, int line, QString function, QString prefix, QString message, const NTFSStream * s);
