// ntfsvolume.h
// Part of SalvageNTFS
// Copyright (C) Will Glynn
//////////////////////////////////////////////////////////////////////////////
// LICENSE
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License (GPL)
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// To read the license please visit http://www.gnu.org/copyleft/gpl.html

#ifndef NTFSVOLUME_H
#define NTFSVOLUME_H

#include "ntfsvolumeinfo.h"
#include "ntfsstream.h"
#include <qptrlist.h>

class NTFSRunList;
class NTFSFile;

class NTFSVolume
{
public:
	friend class NTFSStream;

	NTFSVolume();
	~NTFSVolume();

	// Attach a Volume to a file pointer
	bool attach(int device);

	// Detach the volume from the IO device
	void detach();

	// Attach stream to the data at 'pos' with length 'len'
	bool streamData(NTFSStream *stream, NT64 pos, NT32 length);
	// Attach stream to the data at 'record'
	inline bool streamRecord(NTFSStream *stream, NT64 record)
		{ return streamData(stream, m_info.getPositionForRecord(record), m_info.bytesPerRecord); };
	// Attach stream to the data at 'lcn'
	inline bool streamLcn(NTFSStream *stream, NT64 lcn)
		{ return streamData(stream, m_info.getPositionForLcn(lcn), m_info.bytesPerCluster); };

	// Create stream to the data at 'pos' with length 'len'
	NTFSStream *streamData(NT64 pos, NT32 length);
	// Create stream for the data at 'record'
	inline NTFSStream * streamRecord(NT64 record)
		{ return streamData(m_info.getPositionForRecord(record), m_info.bytesPerRecord); };
	// Create stream for the data at 'lcn'
	inline NTFSStream * streamLcn(NT64 lcn)
		{ return streamData(m_info.getPositionForLcn(lcn), m_info.bytesPerCluster); };

	// Copies data from pos with length to file descriptor fd
	bool copyData(NT64 pos, NT64 length, int fd);

	// Returns a FILE * (or NULL) at record number 'record'
	NTFSFile * getFile(NT64 record);

	// Returns a FILE * (or NULL) using reference number 'refNum'
	inline NTFSFile * getFileByReference(NT64 refNum)
		{ return getFile(m_info.getRecordForReference(refNum)); };

	// Analyze boot sector: identification, validation, and finally filling in m_info with updated info
	bool analyzeBootSector(NT64 sector);

	// Get the volume information
	inline const NTFSVolumeInfo * getVolInfo() { return &m_info; };

protected:
	int m_device;
	NTFSVolumeInfo m_info;

	struct MAPPEDCLUSTER
	{
		NT64 pos;
		NT32 length;
		char * base;
		QPtrList <NTFSStream> children;
	};
	QPtrList <MAPPEDCLUSTER> cachedBlocks;

	char * cache(NT64 pos, NT32 length);
	void manageCache(NT16 cacheEntries = 32);
	void removeChildStream(NTFSStream * s);
	void addChildStream(NTFSStream * s);

	void * mapData(NT64 offset, NT32 length);
	void unmapData(void * addr, NT32 length);

	bool useMmap;
};

#endif
