// ntfsvolumeinfo.h
// Part of SalvageNTFS
// Copyright (C) Will Glynn

#ifndef NTFSVOLUMEINFO_H
#define	NTFSVOLUMEINFO_H

#include "ntfs.h"
#include <qstring.h>

class NTFSVolumeInfo
{
public:
	NTFSVolumeInfo();
	NTFSVolumeInfo(NTFSVolumeInfo &c);

	QString toString();

	// Recalculate computed values -- call after changing anything
	void recalculate();

	// Read from boot sector
	NT16 bytesPerSector;
	NT16 sectorsPerCluster;
	NT64 sectorCount;
	NT64 serialNumber;
	NT64 offsetToMft;
	NT64 offsetToMftMirr;

	// Processed from boot sector
	NT32 bytesPerRecord;
	NT32 bytesPerIndex;

	// Computed values
	NT32 bytesPerCluster;

	inline NT64 getPositionForLcn(NT64 lcn) const { return lcn << shiftLcnToPos; };
	inline NT64 getPositionForRecord(NT64 record) const { return record << shiftRecordToPos; };

	inline NT64 getLcnForPosition(NT64 pos) const { return pos >> shiftLcnToPos; };
	inline NT64 getRecordForPosition(NT64 pos) const { return pos >> shiftRecordToPos; };

	inline NT32 getOffsetFromLcn(NT64 pos) const { return ((NT32)pos) & maskLcnOffset; };
	inline NT32 getOffsetFromRecord(NT64 pos) const { return ((NT32)pos) & maskRecordOffset; };

	NT64 getRecordForReference(NT64 refnum) const;

protected:
	// Shift values
	NT32 shiftLcnToPos;
	NT32 shiftRecordToPos;

	// Masks
	NT32 maskRecordOffset;
	NT32 maskLcnOffset;
};

#endif
