// ls.cpp
// Part of SalvageNTFS
// Copyright (C) Will Glynn
//////////////////////////////////////////////////////////////////////////////
// LICENSE
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License (GPL)
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// To read the license please visit http://www.gnu.org/copyleft/gpl.html

#include <stdio.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include <qstring.h>

#include "ntfsvolume.h"
#include "ntfsutil.h"
#include "ntfsstream.h"
#include "ntfsfile.h"
#include "ntfsvolumeinfo.h"
#include "ntfsindex.h"

void recover(int fd, NT64 record)
{
	// Make a volume attached to our file descriptor
	NTFSVolume v;
	v.attach(fd);

	// Tell it to look at boot sector
	//v.analyzeBootSector(0);

	// Get a FILE for the passed record
	NTFSFile * f = v.getFile(record);

	if (f == NULL)
	{
		// Can't do it, sorry.
		printf("Unable to obtain requested FILE record.\n");
		return;
	}

	// Got a file

	// De-reference $ATTRIBUTE_LIST, if any
	if (!f->a.deReference(&v))
	{
		printf("Unable to de-reference $ATTRIBUTE_LIST.\n");
		return;
	}

	// Attach an index
	NTFSIndex i;
	if (!i.read(f, &v))
	{
		printf("Failed to read directory. FILE record looks like:\n%s\n", f->toString().latin1());
	}
	else
	{
		// Dump entries
		printf("%s\n", i.toString().latin1());
	}

	// Clean up
	delete f;
}

// Entry point
int main(int argc, char ** argv)
{
	printf("SalvageNTFS Data Dump\nWritten by Will Glynn\n\n");

	if (argc != 3)
	{
		printf("Usage: %s volume record\n", argv[0]);
		return 1;
	}

	int fd = open(argv[1], O_RDONLY | O_LARGEFILE);

	if (fd == -1)
	{
		printf("Unable to open %s for reading.\n", argv[1]);
		return 1;
	}

	NT64 record = 0;
	if (argc > 2)
		record = toNum(argv[2]);

	recover(fd, record);

	if (close(fd) == -1)
	{
		printf("Unable to close the volume file.\n");
	}

	return 0;
}
