// scanrecover/scanrecover.cpp
// Part of SalvageNTFS
// Copyright (C) Will Glynn
//////////////////////////////////////////////////////////////////////////////
// LICENSE
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License (GPL)
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// To read the license please visit http://www.gnu.org/copyleft/gpl.html


#include <stdio.h>

#include "ntfsvolume.h"
#include "ntfsutil.h"
#include "ntfsstream.h"
#include "ntfsfile.h"
#include "ntfsvolumeinfo.h"
#include "ntfsrecovery.h"

// Guts of scan/recover
// Based off of ntfsdump
void scan(QString volumeFile, int fd, NT64 sibling, NT64 offset, NT64 length)
{
	NTFSVolume v;
	v.attach(fd);

	// Scan first sector for an NTFS boot sector
	//v.analyzeBootSector(0);

	// Prep testing environment
	NTFSFile * nf;
	NTFSRecovery recovery;
	recovery.open(volumeFile);

	NT64 records = offset;
	NT64 maxRecords = (length ? offset + length : 0);
	
	NT64 targetParentRef = 0;
	
	// Read sibling's file record
	nf = v.getFile(sibling);
	if (!nf || ! nf->isValid())
	{
		// sibling is bad
		printf("Unable to read sibling record.\n");
		return;
	}
	else
	{
		// Read $FILE_NAME, fish out parent reference
		NTFSAttributeDataFilename * fn = (NTFSAttributeDataFilename *)nf->a.getAttribute(0x30);
		if (!fn)
		{
			printf("Sibling record lacks $FILE_NAME attribute. Terminating.\n");
			return;
		}
		
		targetParentRef = fn->parentRef;
	}

	if (length)
	{
		printf(QString("Scanning %1 record(s) from %2.\n").arg(toHex(length)).arg(toHex(offset)));
	}
	else
	{
		printf(QString("Scanning records from %1 to the end of the volume.\n").arg(toHex(offset)));
	}

	bool readSomething = false;
	while ((length == 0) || (records < maxRecords))
	{
		nf = v.getFile(records);
		if (nf && nf->isValid())
		{
			readSomething = true;
			
			// Get the $FILE_NAME attribute 
			NTFSAttributeDataFilename * fn = (NTFSAttributeDataFilename *)nf->a.getAttribute(0x30);
			
			// Check parent reference
			if (fn && fn->parentRef == targetParentRef)
			{
				// Hit!
				printf(QString("Record %1 matches desired parent reference. Recovering.\n").arg(toHex(records)));
				recovery.recoverFile(records);
			}
		}
		delete nf;

		records ++;
	}

	if (!readSomething)
	{
		printf("No FILE records found.\n");
	}
}

// Entry point
int main(int argc, char ** argv)
{
	printf("SalvageNTFS Scan/Recover\nWritten by Will Glynn\n\n");

	if (argc < 3 || argc > 5)
	{
		printf("Usage: %s volume parent [offset [length]]\n", argv[0]);
		return 1;
	}

	FILE * f = fopen(argv[1], "r");
	if (!f)
	{
		printf("Unable to open %s for reading.\n", argv[1]);
		return 1;
	}

	NT64 parent, offset = 0, length = 0;
	parent = toNum(argv[2]);
	if (argc > 3)
	{
		offset = toNum(argv[3]);
	}
	if (argc > 4)
	{
		length = toNum(argv[4]);
	}

	scan(argv[1], fileno(f), parent, offset, length);

	return 0;
}
